unit CardBackForm;
//Selects a CardBack from all (1..9) options available.
interface

uses
  Windows, Classes, Controls, Messages, SysUtils, Graphics, Forms,
  Dialogs, CardTable;

type
  TCardBackForm = class(TForm)
    procedure FormActivate(Sender: TObject);
    procedure CardClicked(ACard: TCard; Button: TMouseButton);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent; const Dir: String); reintroduce;
    procedure DrawBacks(AOwner: TComponent; const DeckType: String);
  end;

implementation

{$R *.dfm}

var
  CardTable: TCardTable;
  SearchDir: string;

constructor TCardBackForm.Create(AOwner: TComponent; const Dir: String);
//Dir points to the cardsets directory.
begin
  inherited Create(AOwner);
  SearchDir := Dir;
end;

procedure TCardBackForm.FormActivate(Sender: TObject);
begin
  CardTable.Show;
end;

procedure TCardBackForm.CardClicked(ACard: TCard;
  Button: TMouseButton);
//Returns value of back chosen 1-9.
begin
  ModalResult := ACard.NumericValue+ord(ACard.Suit)*10;
end;

procedure TCardBackForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  CardTable.Free;
end;

procedure TCardBackForm.DrawBacks(AOwner: TComponent; const DeckType: String);
var
  i,j: integer;
  Card: TCard;
  Back: Integer;
  CardValue: TCardValue;
  CardSuit: TCardSuit;
  S: string;
begin
//Creating a cardtable. We know the location of the cardsets. But there is no
//way to inform cardtable of this & we could be in any directory! But if the
//current dir is set to the dir that contains the cardsets all should be ok.
  for i := length(SearchDir) downto 1 do
    if IsPathDelimiter(SearchDir, i) then break;
  SetLength(SearchDir, i-1);
  SetLength(S, MAX_PATH);
  SetLength(S, GetCurrentDirectory(MAX_PATH, PChar(S)));
  if not SameText(S, SearchDir) then
    SetCurrentDirectory(PChar(SearchDir));
  CardTable := TCardTable.Create(Self);
  CardTable.Hide;
  CardTable.CardDeck.DeckName := DeckType;
  CardTable.BufferFix := False;
  CardTable.Color := clSilver;
  CardTable.OnCardClickEvent := CardClicked;
  i := ((CardTable.CardDeck.NoOfCardBacks - 1) Div 3) + 1;
  CardTable.Width := CardTable.CardDeck.CardWidth * 3 + 10;
  CardTable.Height := CardTable.CardDeck.CardHeight * i + 24;
  //Note default CardTable.MinWidth & Height MUST be specified here as the
  //defaults will mess up big time.
  CardTable.MinWidth := 0;
  CardTable.MinHeight := 0;
  Width := CardTable.Width + 10;
  Height := CardTable.Height + 20;
  Back := 1;
  CardValue := Low(TCardValue);
  CardSuit := Low(TCardSuit);
  for j:=0 to 2 do
    for i := 0 to 2 do
    begin
      Card := CardTable.CardDeck.Draw;
      //Value & Suit indicate Back chosen.
      Card.Value := CardValue;
      Card.Suit := CardSuit;
      if CardValue <> cvTen then
        inc(CardValue)
      else begin
        CardValue := Low(TCardValue);
        inc(CardSuit);
      end;
      if CardTable.CardDeck.CardBack = Back then
        CardTable.PlaceCard(Card,8+i*CardTable.CardDeck.CardWidth,
          8+J*CardTable.CardDeck.CardHeight);
      if Back <> 9 then
      begin
        inc(Back);
        CardTable.CardDeck.CardBack := Back;
      end;
    end;
  if not SameText(S, SearchDir) then//Switch back to original dir.
    SetCurrentDirectory(PChar(S));
end;

end.
