unit CEAuxInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem, CEMediaInfoCommon;

const
  AUX_SUPPORT_NUM = 2;
  AuxCapsSupportStr : array[0..AUX_SUPPORT_NUM-1] of string = ('Supports volume control.',
                      'Supports separate left and right volume control.');
  AuxTechnology : array[0..1] of string = ('Audio output from auxiliary input jacks.',
                      'Audio output from an internal CD-ROM drive.');

type
  TAuxSupports = array[0..1] of TFeature;

  TAuxDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    AuxDevList : TList;
    procedure   FillAuxDevList;
    procedure   FillSupports(Supports : dword; var AAuxSupports : TAuxSupports);
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;
    function    GetTechnology(ATechnology : word) : string;

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ATechnology   : string;   // 
                              var ASupports     : TAuxSupports  // 
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;

implementation

constructor TAuxDevices.Create;
begin
  inherited Create;
  AuxDevList:=TList.Create;
  FNumberOfDevices:=auxGetNumDevs;
  FillAuxDevList;
end;

destructor TAuxDevices.Destroy;
begin
  ClearList;
  AuxDevList.Free;
  inherited Destroy;
end;

procedure TAuxDevices.ClearList;
var
  PAUX  : PAuxCaps;
begin
  while AuxDevList.Count>0 do begin
   PAUX:=PAuxCaps(AuxDevList.Items[AuxDevList.Count-1]);
   Dispose(PAUX);
   AuxDevList.Delete(AuxDevList.Count-1);
  end;
end;

function TAuxDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TAuxCaps(AuxDevList.Items[Index]^).wMid);
end;

function TAuxDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TAuxCaps(AuxDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TAuxCaps(AuxDevList.Items[Index]^).vDriverVersion)));
end;

procedure TAuxDevices.FillAuxDevList;
var
  i     : integer;
  PAUX  : PAuxCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PAUX);
      auxGetDevCaps(i,PAUX,SizeOf(TAuxCaps));
      AuxDevList.Add(PAUX);
    end;
  end;
end;

procedure TAuxDevices.FillSupports(Supports : dword; var AAuxSupports : TAuxSupports);
begin
  AAuxSupports[0].FName:=AuxCapsSupportStr[0];
  AAuxSupports[0].FValue:=(Supports and AUXCAPS_VOLUME)<>0;

  AAuxSupports[1].FName:=AuxCapsSupportStr[1];
  AAuxSupports[1].FValue:=(Supports and AUXCAPS_LRVOLUME)<>0;
end;

function TAuxDevices.GetTechnology(ATechnology : word) : string;
begin
  Result:='';
  case ATechnology of
    AUXCAPS_AUXIN   : Result:=AuxTechnology[0];
    AUXCAPS_CDAUDIO : Result:=AuxTechnology[1];
  end;
end;

procedure TAuxDevices.GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ATechnology   : string;   // 
                              var ASupports     : TAuxSupports  // 
                              );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TAuxCaps(AuxDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TAuxCaps(AuxDevList.Items[Index]^).wPid;
  AProductName:=TAuxCaps(AuxDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
  ATechnology:=GetTechnology(TAuxCaps(AuxDevList.Items[index]^).wTechnology);
  FillSupports(TAuxCaps(AuxDevList.Items[Index]^).dwSupport,ASupports);
end;

end.
