unit CEJoyInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem, CEMediaInfoCommon;

const
  JOY_CAP_NUM = 7;
  JoyCapsStr : array[0..JOY_CAP_NUM-1] of string = (
                'Joystick has z-coordinate information.',
                'Joystick has rudder (fourth axis) information.',
                'Joystick has u-coordinate (fifth axis) information.',
                'Joystick has v-coordinate (sixth axis) information.',
                'Joystick has point-of-view information.',
                'Joystick point-of-view supports discrete values.',
                'Joystick point-of-view supports continuous degree bearings.');


type
  TJoyCapsSupport = array[0..6] of TFeature;

  TJoyDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    JoyDevList : TList;
    procedure   FillJoyDevList;
    function    GetVendorName(Index : integer) : string;
    procedure   FillCaps(Caps : cardinal; var AJoyCaps : TJoyCapsSupport);

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var AXmin         : cardinal;
                              var AXmax         : cardinal;
                              var AYmin         : cardinal;
                              var AYmax         : cardinal;
                              var AZmin         : cardinal;
                              var AZmax         : cardinal;
                              var ANumBtn       : cardinal;
                              var APeriodMin    : cardinal;
                              var APeriodMax    : cardinal;
                              var ARmin         : cardinal;
                              var ARmax         : cardinal;
                              var AUmin         : cardinal;
                              var AUmax         : cardinal;
                              var AVmin         : cardinal;
                              var AVmax         : cardinal;
                              var ACaps         : TJoyCapsSupport;
                              var AMaxAxes      : cardinal;
                              var ANumAxes      : cardinal;
                              var AMaxBtn       : cardinal;
                              var ARegKey       : string;
                              var AOEMVxD       : string
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;


implementation

constructor TJoyDevices.Create;
var
  JoyNum  : integer;
  i       : integer;
  JoyInfo : TJoyInfo;
begin
  inherited Create;
  JoyDevList:=TList.Create;
  FNumberOfDevices:=0;
  JoyNum:=joyGetNumDevs;
  if JoyNum>0 then begin
    for i:=0 to JoyNum-1 do begin
      try
        if joyGetPos(i,@JoyInfo)=JOYERR_NOERROR then Inc(FNumberOfDevices);
      except
      end;  
    end;
  end;
  FillJoyDevList;
end;

destructor TJoyDevices.Destroy;
begin
  ClearList;
  JoyDevList.Free;
  inherited Destroy;
end;

procedure TJoyDevices.ClearList;
var
  PJOY  : PJoyCaps;
begin
  while JoyDevList.Count>0 do begin
   PJOY:=PJoyCaps(JoyDevList.Items[JoyDevList.Count-1]);
   Dispose(PJOY);
   JoyDevList.Delete(JoyDevList.Count-1);
  end;
end;

function TJoyDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TJoyCaps(JoyDevList.Items[Index]^).wMid);
end;

procedure TJoyDevices.FillJoyDevList;
var
  i     : integer;
  PJOY  : PJoyCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PJOY);
      joyGetDevCaps(i,PJOY,SizeOf(TJoyCaps));
      JoyDevList.Add(PJOY);
    end;
  end;
end;

procedure TJoyDevices.FillCaps(Caps : cardinal; var AJoyCaps : TJoyCapsSupport);
begin
  AJoyCaps[0].FName:=JoyCapsStr[0];
  AJoyCaps[0].FValue:=(Caps and JOYCAPS_HASZ)<>0;

  AJoyCaps[1].FName:=JoyCapsStr[1];
  AJoyCaps[1].FValue:=(Caps and JOYCAPS_HASR)<>0;

  AJoyCaps[2].FName:=JoyCapsStr[2];
  AJoyCaps[2].FValue:=(Caps and JOYCAPS_HASU)<>0;

  AJoyCaps[3].FName:=JoyCapsStr[3];
  AJoyCaps[3].FValue:=(Caps and JOYCAPS_HASV)<>0;

  AJoyCaps[4].FName:=JoyCapsStr[4];
  AJoyCaps[4].FValue:=(Caps and JOYCAPS_HASPOV)<>0;

  AJoyCaps[5].FName:=JoyCapsStr[5];
  AJoyCaps[5].FValue:=(Caps and JOYCAPS_POV4DIR)<>0;

  AJoyCaps[6].FName:=JoyCapsStr[6];
  AJoyCaps[6].FValue:=(Caps and JOYCAPS_POVCTS)<>0;
end;

procedure TJoyDevices.GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var AXmin         : cardinal;
                              var AXmax         : cardinal;
                              var AYmin         : cardinal;
                              var AYmax         : cardinal;
                              var AZmin         : cardinal;
                              var AZmax         : cardinal;
                              var ANumBtn       : cardinal;
                              var APeriodMin    : cardinal;
                              var APeriodMax    : cardinal;
                              var ARmin         : cardinal;
                              var ARmax         : cardinal;
                              var AUmin         : cardinal;
                              var AUmax         : cardinal;
                              var AVmin         : cardinal;
                              var AVmax         : cardinal;
                              var ACaps         : TJoyCapsSupport;
                              var AMaxAxes      : cardinal;
                              var ANumAxes      : cardinal;
                              var AMaxBtn       : cardinal;
                              var ARegKey       : string;
                              var AOEMVxD       : string
                              );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TJoyCaps(JoyDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TJoyCaps(JoyDevList.Items[Index]^).wPid;
  AProductName:=TJoyCaps(JoyDevList.Items[Index]^).szPname;
  AXmin:=TJoyCaps(JoyDevList.Items[Index]^).wXmin;
  AXmax:=TJoyCaps(JoyDevList.Items[Index]^).wXmax;
  AYmin:=TJoyCaps(JoyDevList.Items[Index]^).wYmin;
  AYmax:=TJoyCaps(JoyDevList.Items[Index]^).wYmax;
  AZmin:=TJoyCaps(JoyDevList.Items[Index]^).wZmin;
  AZmax:=TJoyCaps(JoyDevList.Items[Index]^).wZmax;
  ANumBtn:=TJoyCaps(JoyDevList.Items[Index]^).wNumButtons;
  APeriodMin:=TJoyCaps(JoyDevList.Items[Index]^).wPeriodMin;
  APeriodMax:=TJoyCaps(JoyDevList.Items[Index]^).wPeriodMax;
  ARmin:=TJoyCaps(JoyDevList.Items[Index]^).wRmin;
  ARmax:=TJoyCaps(JoyDevList.Items[Index]^).wRmax;
  AUmin:=TJoyCaps(JoyDevList.Items[Index]^).wUmin;
  AUmax:=TJoyCaps(JoyDevList.Items[Index]^).wUmax;
  AVmin:=TJoyCaps(JoyDevList.Items[Index]^).wVmin;
  AVmax:=TJoyCaps(JoyDevList.Items[Index]^).wVmax;
  FillCaps(TJoyCaps(JoyDevList.Items[Index]^).wCaps,ACaps);
  AMaxAxes:=TJoyCaps(JoyDevList.Items[Index]^).wMaxAxes;
  ANumAxes:=TJoyCaps(JoyDevList.Items[Index]^).wNumAxes;
  AMaxBtn:=TJoyCaps(JoyDevList.Items[Index]^).wMaxButtons;
  ARegKey:=TJoyCaps(JoyDevList.Items[Index]^).szRegKey;
  AOEMVxD:=TJoyCaps(JoyDevList.Items[Index]^).szOEMVxD;
end;


end.
