unit CEMidiInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem, CEMediaInfoCommon;

const
  MIDI_SUPPORT_NUM  = 4;
  MidiCapsSupportStr : array[0..MIDI_SUPPORT_NUM-1] of string = ('Supports patch caching.',
                        'Supports separate left and right volume control.',
                        'Provides direct support for the midiStreamOut function.',
                        'Supports volume control.');
  MidiTechnology    : array[0..4] of string = ('The device is an FM synthesizer.',
                        'The device is the Microsoft MIDI mapper.',
                        'The device is a MIDI hardware port.',
                        'The device is a square wave synthesizer.',
                        'The device is a synthesizer.');

type
  TMidiOutSupports = array[0..3] of TFeature;

  TMidiOutDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    MidiOutDevList : TList;
    procedure   FillMidiOutDevList;
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;
    procedure   FillSupports(Supports : dword; var AMidiOutSupports : TMidiOutSupports);
    function    GetTechnology(ATechnology : word) : string;

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ATechnology   : string;   // 
                              var AVoices       : word;     //  
                              var ANotes        : word;     //  
                              var AChannelMask  : word;     //
                              var ASupport      : TMidiOutSupports//  
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;

  TMidiInDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    MidiInDevList : TList;
    procedure   FillMidiInDevList;
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string   //  
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;

implementation

//======================= TMidiOutDevices ======================//
constructor TMidiOutDevices.Create;
begin
  inherited Create;
  MidiOutDevList:=TList.Create;
  FNumberOfDevices:=midiOutGetNumDevs;
  FillMidiOutDevList;
end;

destructor  TMidiOutDevices.Destroy;
begin
  ClearList;
  MidiOutDevList.Free;
  inherited Destroy;
end;

procedure TMidiOutDevices.ClearList;
var
  PMOC  : PMidiOutCaps;
begin
  while MidiOutDevList.Count>0 do begin
   PMOC:=PMidiOutCaps(MidiOutDevList.Items[MidiOutDevList.Count-1]);
   Dispose(PMOC);
   MidiOutDevList.Delete(MidiOutDevList.Count-1);
  end;
end;

procedure TMidiOutDevices.FillMidiOutDevList;
var
  i     : integer;
  PMOC  : PMidiOutCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PMOC);
      midiOutGetDevCaps(i,PMOC,SizeOf(TMidiOutCaps));
      MidiOutDevList.Add(PMOC);
    end;
  end;
end;

function TMidiOutDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TMidiOutCaps(MidiOutDevList.Items[Index]^).wMid);
end;

function TMidiOutDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TMidiOutCaps(MidiOutDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TMidiOutCaps(MidiOutDevList.Items[Index]^).vDriverVersion)));
end;

procedure TMidiOutDevices.FillSupports(Supports : dword; var AMidiOutSupports : TMidiOutSupports);
begin
  AMidiOutSupports[0].FName:=MidiCapsSupportStr[0];
  AMidiOutSupports[0].FValue:=(Supports and MIDICAPS_CACHE)<>0;

  AMidiOutSupports[1].FName:=MidiCapsSupportStr[1];
  AMidiOutSupports[1].FValue:=(Supports and MIDICAPS_LRVOLUME)<>0;

  AMidiOutSupports[2].FName:=MidiCapsSupportStr[2];
  AMidiOutSupports[2].FValue:=(Supports and MIDICAPS_STREAM)<>0;

  AMidiOutSupports[3].FName:=MidiCapsSupportStr[3];
  AMidiOutSupports[3].FValue:=(Supports and MIDICAPS_VOLUME)<>0;
end;

function  TMidiOutDevices.GetTechnology(ATechnology : word) : string;
begin
  Result:='';
  case ATechnology of
    MOD_MIDIPORT  : Result:=MidiTechnology[2];
    MOD_SYNTH     : Result:=MidiTechnology[4];
    MOD_SQSYNTH   : Result:=MidiTechnology[3];
    MOD_FMSYNTH   : Result:=MidiTechnology[0];
    MOD_MAPPER    : Result:=MidiTechnology[1];
  end;
end;

procedure TMidiOutDevices.GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ATechnology   : string;   // 
                              var AVoices       : word;     //  
                              var ANotes        : word;     //  
                              var AChannelMask  : word;     //
                              var ASupport      : TMidiOutSupports//  
                              );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TMidiOutCaps(MidiOutDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TMidiOutCaps(MidiOutDevList.Items[Index]^).wPid;
  AProductName:=TMidiOutCaps(MidiOutDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
  ATechnology:=GetTechnology(TMidiOutCaps(MidiOutDevList.Items[index]^).wTechnology);
  AVoices:=TMidiOutCaps(MidiOutDevList.Items[Index]^).wVoices;
  ANotes:=TMidiOutCaps(MidiOutDevList.Items[Index]^).wNotes;
  AChannelMask:=TMidiOutCaps(MidiOutDevList.Items[Index]^).wChannelMask;
  FillSupports(TMidiOutCaps(MidiOutDevList.Items[Index]^).dwSupport,ASupport);
end;

//======================= TMidiInDevices ======================//
constructor TMidiInDevices.Create;
begin
  inherited Create;
  MidiInDevList:=TList.Create;
  FNumberOfDevices:=MidiInGetNumDevs;
  FillMidiInDevList;
end;

destructor  TMidiInDevices.Destroy;
begin
  ClearList;
  MidiInDevList.Free;
  inherited Destroy;
end;

procedure TMidiInDevices.ClearList;
var
  PMIC  : PMidiInCaps;
begin
  while MidiInDevList.Count>0 do begin
   PMIC:=PMidiInCaps(MidiInDevList.Items[MidiInDevList.Count-1]);
   Dispose(PMIC);
   MidiInDevList.Delete(MidiInDevList.Count-1);
  end;
end;

procedure TMidiInDevices.FillMidiInDevList;
var
  i     : integer;
  PMIC  : PMidiInCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PMIC);
      MidiInGetDevCaps(i,PMIC,SizeOf(TMidiInCaps));
      MidiInDevList.Add(PMIC);
    end;
  end;
end;

function TMidiInDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TMidiInCaps(MidiInDevList.Items[Index]^).wMid);
end;

function TMidiInDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TMidiInCaps(MidiInDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TMidiInCaps(MidiInDevList.Items[Index]^).vDriverVersion)));
end;

procedure TMidiInDevices.GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string   //  
                              );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TMidiInCaps(MidiInDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TMidiInCaps(MidiInDevList.Items[Index]^).wPid;
  AProductName:=TMidiInCaps(MidiInDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
end;



end.
