unit CEMixerInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem, CEMediaInfoCommon;

type
  TMixerDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    MixerDevList : TList;
    procedure   FillMixerDevList;
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //   
                              var AVendorName   : string;   //  
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ADestinations : dword
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;

implementation

constructor TMixerDevices.Create;
begin
  inherited Create;
  MixerDevList:=TList.Create;
  FNumberOfDevices:=MixerGetNumDevs;
  FillMixerDevList;
end;

destructor TMixerDevices.Destroy;
begin
  ClearList;
  MixerDevList.Free;
  inherited Destroy;
end;

procedure TMixerDevices.ClearList;
var
  PMIX  : PMixerCaps;
begin
  while MixerDevList.Count>0 do begin
   PMIX:=PMixerCaps(MixerDevList.Items[MixerDevList.Count-1]);
   Dispose(PMIX);
   MixerDevList.Delete(MixerDevList.Count-1);
  end;
end;

function TMixerDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TMixerCaps(MixerDevList.Items[Index]^).wMid);
end;

function TMixerDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TMixerCaps(MixerDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TMixerCaps(MixerDevList.Items[Index]^).vDriverVersion)));
end;

procedure TMixerDevices.FillMixerDevList;
var
  i     : integer;
  PMIX  : PMixerCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PMIX);
      MixerGetDevCaps(i,PMIX,SizeOf(TMixerCaps));
      MixerDevList.Add(PMIX);
    end;
  end;
end;

procedure TMixerDevices.GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //   
                              var AVendorName   : string;   //  
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var ADestinations : dword
                              );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TMixerCaps(MixerDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TMixerCaps(MixerDevList.Items[Index]^).wPid;
  AProductName:=TMixerCaps(MixerDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
  ADestinations:=TMixerCaps(MixerDevList.Items[Index]^).cDestinations;
end;

end.
