unit CEWaveInfo;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MMSystem, CEMediaInfoCommon;

const
  WAVE_FORMAT_NUM   = 12;
  WaveCapsFormatStr : array[0..WAVE_FORMAT_NUM-1] of string = (
                      '11.025 kHz, mono, 8-bit',
                      '11.025 kHz, mono, 16-bit',
                      '11.025 kHz, stereo, 8-bit',
                      '11.025 kHz, stereo, 16-bit',
                      '22.05 kHz, mono, 8-bit',
                      '22.05 kHz, mono, 16-bit',
                      '22.05 kHz, stereo, 8-bit',
                      '22.05 kHz, stereo, 16-bit',
                      '44.1 kHz, mono, 8-bit',
                      '44.1 kHz, mono, 16-bit',
                      '44.1 kHz, stereo, 8-bit',
                      '44.1 kHz, stereo, 16-bit');
  WAVE_SUPPORT_NUM  = 7;
  WaveCapsSupportStr : array[0..WAVE_SUPPORT_NUM-1] of string = (
                      'Supports separate left and right volume control.',
                      'Supports pitch control.',
                      'Supports playback rate control.',
                      'The driver is synchronous and will block while playing a buffer.',
                      'Supports volume control.',
                      'Returns sample-accurate position information.',
                      'Supports DirectSound.');

type
  TWaveFormats      = array[0..11] of TFeature;
  TWaveOutSupports  = array[0..6] of TFeature;

  TWaveOutDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    WaveOutDevList : TList;
    procedure   FillWaveOutDevList;
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;
    procedure   FillSupports(Supports : dword; var AWaveOutSupports : TWaveOutSupports);

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var AFormats      : TWaveFormats; //  
                              var AChannels     : word;     //  
                              var ASupport      : TWaveOutSupports//  
                              );

  published
    property  NumberOfDevices  : integer read FNumberOfDevices write FNumberOfDevices;
  end;

  TWaveInDevices  = class (TPersistent)
  private
    FNumberOfDevices  : integer;

    WaveInDevList : TList;
    procedure   FillWaveInDevList;
    function    GetVendorName(Index : integer) : string;
    function    GetDriverVersion(Index : integer) : string;

    procedure ClearList;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure   GetDeviceInfo(Index             : integer;  //     
                              var AVendorID     : word;     //    
                              var AVendorName   : string;   //   
                              var AProductID    : word;     //   
                              var AProductName  : string;   //  
                              var ADrvVersion   : string;   //  
                              var AFormats      : TWaveFormats; //  
                              var AChannels     : word     //  
                              );
  published
    property  NumberOfDevices : integer read FNumberOfDevices write FNumberOfDevices;
  end;

  procedure FillWaveFormats(Formats : dword; var AWaveFormats : TWaveFormats);

implementation

procedure FillWaveFormats(Formats : dword; var AWaveFormats : TWaveFormats);
begin
  AWaveFormats[0].FName:=WaveCapsFormatStr[0];
  AWaveFormats[0].FValue:=(Formats and WAVE_FORMAT_1M08)<>0;

  AWaveFormats[1].FName:=WaveCapsFormatStr[1];
  AWaveFormats[1].FValue:=(Formats and WAVE_FORMAT_1M16)<>0;

  AWaveFormats[2].FName:=WaveCapsFormatStr[2];
  AWaveFormats[2].FValue:=(Formats and WAVE_FORMAT_1S08)<>0;

  AWaveFormats[3].FName:=WaveCapsFormatStr[3];
  AWaveFormats[3].FValue:=(Formats and WAVE_FORMAT_1S16)<>0;

  AWaveFormats[4].FName:=WaveCapsFormatStr[4];
  AWaveFormats[4].FValue:=(Formats and WAVE_FORMAT_2M08)<>0;

  AWaveFormats[5].FName:=WaveCapsFormatStr[5];
  AWaveFormats[5].FValue:=(Formats and WAVE_FORMAT_2M16)<>0;

  AWaveFormats[6].FName:=WaveCapsFormatStr[6];
  AWaveFormats[6].FValue:=(Formats and WAVE_FORMAT_2S08)<>0;

  AWaveFormats[7].FName:=WaveCapsFormatStr[7];
  AWaveFormats[7].FValue:=(Formats and WAVE_FORMAT_2S16)<>0;

  AWaveFormats[8].FName:=WaveCapsFormatStr[8];
  AWaveFormats[8].FValue:=(Formats and WAVE_FORMAT_4M08)<>0;

  AWaveFormats[9].FName:=WaveCapsFormatStr[9];
  AWaveFormats[9].FValue:=(Formats and WAVE_FORMAT_4M16)<>0;

  AWaveFormats[10].FName:=WaveCapsFormatStr[10];
  AWaveFormats[10].FValue:=(Formats and WAVE_FORMAT_4S08)<>0;

  AWaveFormats[11].FName:=WaveCapsFormatStr[11];
  AWaveFormats[11].FValue:=(Formats and WAVE_FORMAT_1M08)<>0;
end;

//================== TWaveOutDevice ======================//
constructor TWaveOutDevices.Create;
begin
  inherited Create;
  WaveOutDevList:=TList.Create;
  FNumberOfDevices:=waveOutGetNumDevs;
  FillWaveOutDevList;
end;


destructor TWaveOutDevices.Destroy;
begin
  ClearList;
  WaveOutDevList.Free;
  inherited Destroy;
end;

procedure TWaveOutDevices.ClearList;
var
  PWOD  : PWaveOutCaps;
begin
  while WaveOutDevList.Count>0 do begin
   PWOD:=PWaveOutCaps(WaveOutDevList.Items[WaveOutDevList.Count-1]);
   Dispose(PWOD);
   WaveOutDevList.Delete(WaveOutDevList.Count-1);
  end;
end;

procedure TWaveOutDevices.FillWaveOutDevList;
var
  i   : integer;
  PWOD  : PWaveOutCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PWOD);
      waveOutGetDevCaps(i,PWOD,SizeOf(TWaveOutCaps));
      WaveOutDevList.Add(PWOD);
    end;
  end;
end;

function  TWaveOutDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TWaveOutCaps(WaveOutDevList.Items[Index]^).wMid);
end;

function TWaveOutDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TWaveOutCaps(WaveOutDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TWaveOutCaps(WaveOutDevList.Items[Index]^).vDriverVersion)));
end;

procedure TWaveOutDevices.FillSupports(Supports : dword; var AWaveOutSupports : TWaveOutSupports);
begin
  AWaveOutSupports[0].FName:=WaveCapsSupportStr[0];
  AWaveOutSupports[0].FValue:=(Supports and WAVECAPS_LRVOLUME)<>0;

  AWaveOutSupports[1].FName:=WaveCapsSupportStr[1];
  AWaveOutSupports[1].FValue:=(Supports and WAVECAPS_PITCH)<>0;

  AWaveOutSupports[2].FName:=WaveCapsSupportStr[2];
  AWaveOutSupports[2].FValue:=(Supports and WAVECAPS_PLAYBACKRATE)<>0;

  AWaveOutSupports[3].FName:=WaveCapsSupportStr[3];
  AWaveOutSupports[3].FValue:=(Supports and WAVECAPS_SYNC)<>0;

  AWaveOutSupports[4].FName:=WaveCapsSupportStr[4];
  AWaveOutSupports[4].FValue:=(Supports and WAVECAPS_VOLUME)<>0;

  AWaveOutSupports[5].FName:=WaveCapsSupportStr[5];
  AWaveOutSupports[5].FValue:=(Supports and WAVECAPS_SAMPLEACCURATE)<>0;

  AWaveOutSupports[6].FName:=WaveCapsSupportStr[6];
  AWaveOutSupports[6].FValue:=(Supports and WAVECAPS_DIRECTSOUND)<>0;

end;

procedure  TWaveOutDevices.GetDeviceInfo(Index             : integer;  //     
                                         var AVendorID     : word;     //    
                                         var AVendorName   : string;   //   
                                         var AProductID    : word;     //   
                                         var AProductName  : string;   //  
                                         var ADrvVersion   : string;   //  
                                         var AFormats      : TWaveFormats; //  
                                         var AChannels     : word;     //  
                                         var ASupport      : TWaveOutSupports//  
                                         );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TWaveOutCaps(WaveOutDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TWaveOutCaps(WaveOutDevList.Items[Index]^).wPid;
  AProductName:=TWaveOutCaps(WaveOutDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
  FillWaveFormats(TWaveOutCaps(WaveOutDevList.Items[Index]^).dwFormats,AFormats);
  AChannels:=TWaveOutCaps(WaveOutDevList.Items[Index]^).wChannels;
  FillSupports(TWaveOutCaps(WaveOutDevList.Items[Index]^).dwSupport,ASupport);
end;

//================ TWaveInDevices ================//
constructor TWaveInDevices.Create;
begin
  inherited Create;
  WaveInDevList:=TList.Create;
  FNumberOfDevices:=waveInGetNumDevs;
  FillWaveInDevList;
end;

destructor  TWaveInDevices.Destroy;
begin
  ClearList;
  WaveInDevList.Free;
  inherited Destroy;
end;

procedure TWaveInDevices.ClearList;
var
  PWID  : PWaveInCaps;
begin
  while WaveInDevList.Count>0 do begin
   PWID:=PWaveInCaps(WaveInDevList.Items[WaveInDevList.Count-1]);
   Dispose(PWID);
   WaveInDevList.Delete(WaveInDevList.Count-1);
  end;
end;

procedure TWaveInDevices.FillWaveInDevList;
var
  i   : integer;
  PWID  : PWaveInCaps;
begin
  if FNumberOfDevices>0 then begin
    for i:=0 to FNumberOfDevices-1 do begin
      New(PWID);
      waveInGetDevCaps(i,PWID,SizeOf(TWaveInCaps));
      WaveInDevList.Add(PWID);
    end;
  end;
end;

function  TWaveInDevices.GetVendorName(Index : integer) : string;
begin
  Result:='Unknown Vendor';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=DecodeVendorName(TWaveInCaps(WaveInDevList.Items[Index]^).wMid);
end;

function TWaveInDevices.GetDriverVersion(Index : integer) : string;
begin
  Result:='';
  if Index > (FNumberOfDevices-1) then Exit;
  Result:=IntToStr(HiByte(LoWord(TWaveInCaps(WaveInDevList.Items[Index]^).vDriverVersion)))+'.'+
          intToStr(LoByte(LoWord(TWaveInCaps(WaveInDevList.Items[Index]^).vDriverVersion)));
end;

procedure  TWaveInDevices.GetDeviceInfo(Index             : integer;  //     
                                         var AVendorID     : word;     //    
                                         var AVendorName   : string;   //   
                                         var AProductID    : word;     //   
                                         var AProductName  : string;   //  
                                         var ADrvVersion   : string;   //  
                                         var AFormats      : TWaveFormats; //  
                                         var AChannels     : word     //  
                                         );
begin
  if Index > (FNumberOfDevices-1) then Exit;
  AVendorID:=TWaveInCaps(WaveInDevList.Items[Index]^).wMid;
  AVendorName:=GetVendorName(Index);
  AProductID:=TWaveInCaps(WaveInDevList.Items[Index]^).wPid;
  AProductName:=TWaveInCaps(WaveInDevList.Items[Index]^).szPname;
  ADrvVersion:=GetDriverVersion(Index);
  FillWaveFormats(TWaveInCaps(WaveInDevList.Items[Index]^).dwFormats,AFormats);
  AChannels:=TWaveInCaps(WaveInDevList.Items[Index]^).wChannels;
end;

end.
